Package: baySeq
Type: Package
Title: Empirical Bayesian analysis of patterns of differential
        expression in count data
Version: 2.46.0
Date: 2012-12-20
Depends: R (>= 2.3.0), methods
Imports: edgeR, GenomicRanges, abind, parallel, graphics, stats, utils
Suggests: BiocStyle, BiocGenerics
Authors@R: c(person("Thomas J.", "Hardcastle", role = "aut"),
  person("Samuel", "Granjeaud", role = "cre", email = "samuel.granjeaud@inserm.fr", comment=c(ORCID="0000-0001-9245-1535")))
Description: This package identifies differential expression in
        high-throughput 'count' data, such as that derived from
        next-generation sequencing machines, calculating estimated
        posterior likelihoods of differential expression (or more
        complex hypotheses) via empirical Bayesian methods.
License: GPL-3
LazyLoad: yes
biocViews: Sequencing, DifferentialExpression, MultipleComparison,
        SAGE, Bayesian, Coverage
URL: https://github.com/samgg/baySeq
BugReports: https://github.com/samgg/baySeq/issues
RoxygenNote: 7.2.3
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-04-28 13:02:50 UTC
RemoteUrl: https://github.com/bioc/baySeq
RemoteRef: RELEASE_3_23
RemoteSha: 76c0d69b3e7d9ca7c9a2ddbcc1621f521754438c
NeedsCompilation: no
Packaged: 2026-04-29 15:07:32 UTC; root
Author: Thomas J. Hardcastle [aut],
  Samuel Granjeaud [cre] (ORCID: <https://orcid.org/0000-0001-9245-1535>)
Maintainer: Samuel Granjeaud <samuel.granjeaud@inserm.fr>
Built: R 4.6.0; ; 2026-04-29 15:52:20 UTC; unix
